@@ -28,6 +28,42 @@ def yellow(str):
     return f'\033[33m{str}\033[0m'
 
 
+def blue(str):
+    return f'\033[34m{str}\033[0m'
+
+
+def purple(str):
+    return f'\033[35m{str}\033[0m'
+
+
+def cyan(str):
+    return f'\033[36m{str}\033[0m'
+
+
+def lred(str):
+    return f'\033[1;31m{str}\033[0m'
+
+
+def lgreen(str):
+    return f'\033[1;32m{str}\033[0m'
+
+
+def lyellow(str):
+    return f'\033[1;33m{str}\033[0m'
+
+
+def lblue(str):
+    return f'\033[1;34m{str}\033[0m'
+
+
+def lpurple(str):
+    return f'\033[1;35m{str}\033[0m'
+
+
+def lcyan(str):
+    return f'\033[1;36m{str}\033[0m'
+
+
 def now(format='%y%m%d-%H:%M:%S'):
     return datetime.strftime(datetime.now(), format)
 
@@ -115,7 +151,7 @@ class ylog:
     def info(self, *args):
         args = [f'{arg}' for arg in args]
         if not self.filename:
-            _str = f'{now()}|{green("INFO")} {" ".join(args)}'
+            _str = f'{now()}|{lgreen("INFO")} {" ".join(args)}'
         else:
             _str = f'{now()}|{"INFO"} {" ".join(args)}'
         if self.lock is not None:
@@ -127,7 +163,7 @@ class ylog:
     def warn(self, *args):
         args = [f'{arg}' for arg in args]
         if not self.filename:
-            _str = f'{now()}|{yellow("WARN")} {" ".join(args)}'
+            _str = f'{now()}|{lyellow("WARN")} {" ".join(args)}'
         else:
             _str = f'{now()}|{"WARN"} {" ".join(args)}'
         if self.lock is not None:
@@ -139,7 +175,7 @@ class ylog:
     def err(self, *args):
         args = [f'{arg}' for arg in args]
         if not self.filename:
-            _str = f'{now()}|{red("ERROR")} {" ".join(args)}'
+            _str = f'{now()}|{lred("ERROR")} {" ".join(args)}'
         else:
             _str = f'{now()}|{"ERROR"} {" ".join(args)}'
         if self.lock is not None:
@@ -169,40 +205,53 @@ def is_windows():
     return sys.platform == 'win32'
 
 
+def is_docker():
+    return pexist('/.dockerenv')
+
+
 def python():
     if is_windows():
         return 'python'
     return runok('which python3')
 
 
+def replace_home(path):
+    return os.path.expanduser(path)
+
+
 def __run(cmd):
     if is_windows():
         cmd = cmd.replace('sudo', '').strip()
     return sp.getstatusoutput(cmd)
 
 
-def sys_run(cmd):
-    os.system(cmd)
+def sys_run(cmd, show=True) -> int:
+    if show:
+        info(f'{lgreen("-")} {cmd}')
+    return os.system(cmd)
 
 
-def run(cmd) -> str:
-    info(f'{green("-")} {cmd}')
+def run(cmd, show=True) -> str:
+    if show:
+        info(f'{lgreen("-")} {cmd}')
     code, output = __run(cmd)
     if code != 0:
         warn(code, output)
     return output
 
 
-def runex(cmd) -> tuple:
-    info(f'{green("-")} {cmd}')
+def runex(cmd, show=True) -> tuple:
+    if show:
+        info(f'{lyellow("-")} {cmd}')
     code, output = __run(cmd)
     if code != 0:
         warn(code, output)
     return code, output
 
 
-def runok(cmd) -> tuple:
-    info(f'ensure: {cmd}')
+def runok(cmd, show=True) -> tuple:
+    if show:
+        info(f'{lred("-")} {cmd}')
     code, output = __run(cmd)
     if code != 0:
         err(code, output)
@@ -223,10 +272,17 @@ def home():
 
 
 def me():
-    return os.environ["USER"]
+    if 'USER' in os.environ:
+        return os.environ['USER']
+    return run('whoami', show=False)
 
 
-def check_file(file_name, package_name):
+def check_file(file_name):
+    code, _ = runex(f'which {file_name}')
+    return code == 0
+
+
+def check_pkg(file_name, package_name):
     code, output = runex(f'which {file_name}')
     if code != 0:
         warn(f'not found {file_name}, try to install {package_name}')
@@ -251,16 +307,19 @@ def get_pip():
 def import_module(mod, mod_name=None):
     if mod_name is None:
         mod_name = mod
-    try:
-        __import__(mod)
-    except ModuleNotFoundError:
-        warn(f'{mod} not found try to install {mod_name}')
-        runok(f'{get_pip()} install {mod_name}')
-    return __import__(mod)
+
+    def _import_module():
+        try:
+            __import__(mod)
+        except ModuleNotFoundError:
+            warn(f'{mod} not found try to install {mod_name}')
+            runok(f'{get_pip()} install {mod_name}')
+        return __import__(mod)
+    return retry(_import_module, interval=1)
 
 
 def clone(url, depth=1, dst_dir='.'):
-    dst_dir = dst_dir.replace('~', home())
+    dst_dir = replace_home(dst_dir)
     if dst_dir != '.' and pexist(dst_dir):
         run(f'rm -rf {dst_dir}')
 
@@ -277,25 +336,58 @@ def mv(source, dest, args=''):
     runok(f'mv {args} {source} {dest}')
 
 
-def lns(source, dest):
+def lns(source, dest, safe=True):
     # dest -> source
-    dest = dest.replace('~', home())
-    if pexist(dest):
-        backup(dest)
-    source = os.path.realpath(source)
     assert pexist(source), f'{source} not exist'
+    dest = replace_home(dest)
+    if pexist(dest):
+        if safe:
+            backup(dest)
+        else:
+            rm(dest)
+    source = os.path.realpath(source)
     runok(f'ln -s {source} {dest}')
 
 
-def backup(source):
+def sort_by_mtime(files: list):
+    return sorted(files, key=lambda f: os.stat(f).st_ctime)
+
+
+def backup(source, max_count=5):
+    source = replace_home(source)
     if not pexist(source):
         return
-    source = source.replace('~', home())
+    basename = os.path.basename(source)
+    dname = os.path.dirname(source)
+    if '.' in basename:
+        *backup_basename, _type = basename.split('.')
+        backup_basename = '.'.join(backup_basename)
+    else:
+        backup_basename = basename
+        _type = None
+    backup_name = os.path.join(dname, backup_basename)
+    backup_files = []
+    full = True
+    for i in range(1, max_count + 1):
+        if _type is None:
+            new_name = backup_name + f'_{i}'
+        else:
+            new_name = backup_name + f'_{i}.{_type}'
+        if not pexist(new_name):
+            backup_name = new_name
+            full = False
+            break
+        backup_files.append(new_name)
+    if full:
+        backup_files = sort_by_mtime(backup_files)
+        rm(backup_files[0])
+        backup_name = backup_files[0]
+
     if is_windows():
         cmd = 'move'
     else:
         cmd = 'mv'
-    runok(f'{cmd} {source} {source}.bak')
+    runok(f'{cmd} {source} {backup_name}')
 
 
 def curdir():
@@ -315,6 +407,7 @@ def pushd(dir=None):
 
 
 def cd(path, show=True):
+    path = replace_home(path)
     assert pexist(path), f'{path} not exist'
     if os.path.isfile(path):
         os.chdir(dirname(path))
@@ -391,3 +484,9 @@ def get_clipboard():
 def set_clipboard(_str):
     pyperclip = import_module('pyperclip')
     pyperclip.copy(_str)
+
+
+def container_running(container_name):
+    docker = import_module('docker')
+    client = docker.from_env()
+    return container_name in [c.name for c in client.containers.list(all=False)]
